@@ -17,7 +17,6 @@ try:
     import xlwt
 except ImportError:
     class FakeXlwt(object):
-
         def __getattr__(self, *args):
             raise RuntimeError(u'xlwt is not installed!')
     xlwt = FakeXlwt()
@@ -95,6 +94,17 @@ class RecordProxy(object):
         pass
 
 
+class DictToRecordProxy(RecordProxy):
+    """
+    Прокси объект для данных, переданных в виде словаря
+    """
+    def load(self, data):
+        """
+        Копирует все записи словаря в атрибуты объекта
+        """
+        self.__dict__.update(data)
+
+
 class GroupingRecordProvider(object):
 
     '''
@@ -830,42 +840,36 @@ class GroupingRecordModelProvider(GroupingRecordProvider):
                 elif agg_type == 'avg':
                     aggr.append(Avg(agg))
 
+            query = self.get_data().values(field)
             if filter:
-                if aggr:
-                    query = self.get_data().filter(filter).values(
-                        field).annotate(*aggr).annotate(count=Count("id"))
-                else:
-                    query = self.get_data().filter(
-                        filter).values(field).distinct()
+                query = query.filter(filter)
+            if aggr:
+                query = query.annotate(*aggr).annotate(count=Count("id"))
             else:
-                if aggr:
-                    query = self.get_data().values(field).annotate(
-                        *aggr).annotate(count=Count("id"))
-                else:
-                    query = self.get_data().values(field).distinct()
+                query = query.distinct()
 
             # сортировка
             sort_fields = []
             # оставлено для совместимости
-            if isinstance(sorting, dict) and sorting.keys():
-                for sort_field, sort_dir in sorting.iteritems():
-                    #необходимо исключить из сортировки поля, которые не входят в aggr, иначе по ним будет сделана ненужная группировка
+            if sorting:
+                if isinstance(sorting, dict):
+                    pairs = sorting.iteritems()
+                elif isinstance(sorting, list):
+                    pairs = sorting
+                else:
+                    raise TypeError("Sorting must be dict or list!")
+                for sort_field, sort_dir in pairs:
+                    # необходимо исключить из сортировки поля,
+                    # которые не входят в aggr,
+                    # иначе по ним будет сделана ненужная группировка
                     if sort_field in aggregates.keys() or sort_field == field:
                         if sort_field in aggregates.keys():
-                            # необходимо добавить суффикс к полю, которое в аггрегатах, иначе сортировать будет не по тому что надо :)
-                            sort_field = sort_field + '__' + aggregates[sort_field]
-                        if sort_dir == 'DESC':
-                            sort_fields.append('-' + sort_field)
-                        else:
-                            sort_fields.append(sort_field)
-            # изначальная ошибка была передавать сортировку в dict - надо было сразу в list
-            elif isinstance(sorting, list) and sorting:
-                for sort_field, sort_dir in sorting:
-                    #необходимо исключить из сортировки поля, которые не входят в aggr, иначе по ним будет сделана ненужная группировка
-                    if sort_field in aggregates.keys() or sort_field == field:
-                        if sort_field in aggregates.keys():
-                            # необходимо добавить суффикс к полю, которое в аггрегатах, иначе сортировать будет не по тому что надо :)
-                            sort_field = sort_field + '__' + aggregates[sort_field]
+                            # необходимо добавить суффикс к полю,
+                            # которое в аггрегатах,
+                            # иначе сортировать будет не по тому что надо :)
+                            sort_field = sort_field + '__' + (
+                                aggregates[sort_field]
+                            )
                         if sort_dir == 'DESC':
                             sort_fields.append('-' + sort_field)
                         else:
@@ -882,52 +886,56 @@ class GroupingRecordModelProvider(GroupingRecordProvider):
 
 
 class GroupingRecordDataProvider(GroupingRecordProvider):
-
     '''
     Провайдер для массива
     '''
+    def update_aggregates(self, aggregates, aggr_rec, rec):
+        for agg in aggregates.keys():
+            agg_type = aggregates[agg]
+            agg_value = self.getattr(rec, agg)
+            if agg_type == 'sum':
+                aggr_rec[agg] = (
+                    (agg_value + aggr_rec[agg])
+                    if agg in aggr_rec else
+                    agg_value
+                )
+            elif agg_type == 'count':
+                aggr_rec[agg] = 1 + aggr_rec.get(agg, 0)
+            elif agg_type == 'min':
+                aggr_rec[agg] = min(
+                    agg_value,
+                    aggr_rec.get(agg, agg_value)
+                )
+            elif agg_type == 'max':
+                aggr_rec[agg] = max(
+                    agg_value,
+                    aggr_rec.get(agg, agg_value)
+                )
+            elif agg_type == 'avg':
+                aggr_rec[agg] = (
+                    (agg_value + aggr_rec[agg])
+                    if agg in aggr_rec else
+                    agg_value
+                )
+            elif callable(agg_type):
+                aggr_rec[agg] = agg_type(aggr_rec.get(agg), agg_value)
 
-    def reader(self, grouped, offset, level_index, level_keys, begin, end, aggregates, sorting):
-        return self.__read_data(grouped, offset, level_index, level_keys, begin, end, aggregates, sorting)
-
-    def __read_data(self, grouped, offset, level_index, level_keys, begin, end, aggregates, sorting):
+    def reader(
+        self, grouped, offset, level_index, level_keys,
+        begin, end, aggregates, sorting
+    ):
         '''
-        вывод элементов дерева группировок в зависимости от уровня, ключевых элементов и интервала в уровне
+        вывод элементов дерева группировок в зависимости от уровня,
+        ключевых элементов и интервала в уровне
         '''
-        # построим ключ кэша
-        #    cache_key = '%s__%s__%s__%s__%s__%s' % (','.join(grouped), offset, level_index, ','.join(level_keys), begin, end)
-        #    if cache_key in out_cache.keys():
-        #        print 'cached data...........'
-        #        return out_cache[cache_key]
-
-        # print 'out_data(): grouped=%s, offset=%s, level_index=%s,
-        # level_keys=%s, begin=%s, end=%s' % (grouped, offset, level_index,
-        # level_keys, begin, end)
-
-        # специальный режим, когда считается общий итог по всем записям - не важна сортировка и группировка
+        # специальный режим, когда считается общий итог по всем записям
+        # - не важна сортировка и группировка
         if level_index == -1:
             aggr_rec = {}
             count = 0
             for rec in self.get_data():
                 # будем считать агрегаты
-                for agg in aggregates.keys():
-                    agg_type = aggregates[agg]
-                    agg_value = self.getattr(rec, agg)
-                    if agg_type == 'sum':
-                        aggr_rec[agg] = agg_value + (aggr_rec[
-                                                     agg] if agg in aggr_rec else 0)
-                    elif agg_type == 'count':
-                        aggr_rec[agg] = 1 + (aggr_rec[
-                                             agg] if agg in aggr_rec else 0)
-                    elif agg_type == 'min':
-                        aggr_rec[agg] = agg_value if aggr_rec.get(
-                            agg, 0) > agg_value else aggr_rec.get(agg, 0)
-                    elif agg_type == 'max':
-                        aggr_rec[agg] = agg_value if aggr_rec.get(
-                            agg, 0) < agg_value else aggr_rec.get(agg, 0)
-                    elif agg_type == 'avg':
-                        aggr_rec[agg] = agg_value + (aggr_rec[
-                                                     agg] if agg in aggr_rec else 0)
+                self.update_aggregates(aggregates, aggr_rec, rec)
                 count += 1
             item = self.create_record()
             self.setattr(item, 'id', None)
@@ -935,21 +943,23 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
             self.setattr(item, 'lindex', None)
             self.setattr(item, 'count', count)
             if aggr_rec:
-                for agg in aggregates.keys():
+                for agg, aggv in aggregates.iteritems():
                     # для средних - посчитаем среднее
-                    if aggregates[agg] == 'avg':
-                        self.setattr(item, agg, aggr_rec[
-                                     agg] / self.getattr(item, 'count'))
+                    if agg == 'avg':
+                        self.setattr(
+                            item, agg,
+                            aggr_rec[agg] / self.getattr(item, 'count')
+                        )
+                    elif callable(aggv):
+                        self.setattr(
+                            item, agg,
+                            aggv(aggr_rec[agg], None)
+                        )
                     else:
                         self.setattr(item, agg, aggr_rec[agg])
             self.calc(item)
             return item
 
-        # проведем сортировку собранного уровня
-        # if len(sorting.keys()) == 1:
-        #    sorted_data = sorted(self.get_data(), key=lambda k: getattr(k, sorting.keys()[0]), reverse=(sorting.values()[0] == 'DESC'))
-        # else:
-        #    sorted_data = self.get_data()
         # сортировать будем после группировки и фильтрации
         raw_data = self.get_data()
         res = []
@@ -961,7 +971,8 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
             aggregate_values = {}
             prepared = []
 
-            # если берется уровень больший, чем количество группировок, то выдаем просто записи
+            # если берется уровень больший, чем количество группировок,
+            # то выдаем просто записи
             if level_index >= len(grouped):
                 field = None
             else:
@@ -987,31 +998,16 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
                             aggregate_values[group_value] = aggr_rec
                             prepared.append((group_value, rec))
                         else:
-                            level[group_value] = level[group_value] + 1
+                            level[group_value] += 1
                             aggr_rec = aggregate_values[group_value]
                         # будем считать агрегаты
-                        for agg in aggregates.keys():
-                            agg_type = aggregates[agg]
-                            agg_value = self.getattr(rec, agg)
-                            if agg_type == 'sum':
-                                aggr_rec[agg] = agg_value + (aggr_rec[
-                                                             agg] if agg in aggr_rec else 0)
-                            elif agg_type == 'count':
-                                aggr_rec[agg] = 1 + (aggr_rec[
-                                                     agg] if agg in aggr_rec else 0)
-                            elif agg_type == 'min':
-                                aggr_rec[agg] = agg_value if aggr_rec.get(
-                                    agg, 0) > agg_value else aggr_rec.get(agg, 0)
-                            elif agg_type == 'max':
-                                aggr_rec[agg] = agg_value if aggr_rec.get(
-                                    agg, 0) < agg_value else aggr_rec.get(agg, 0)
-                            elif agg_type == 'avg':
-                                aggr_rec[agg] = agg_value + (aggr_rec[
-                                                             agg] if agg in aggr_rec else 0)
+                        self.update_aggregates(aggregates, aggr_rec, rec)
                     else:
                         prepared.append((rec, rec))
             # теперь выведем запрошенные элементы уровня
-            # придется обработать все записи уровня, т.к. требуется еще отсортировать их и лишь потом ограничить количество
+            # придется обработать все записи уровня,
+            # т.к. требуется еще отсортировать их,
+            # и лишь потом ограничить количество
             for i, rec in prepared:
                 if field:
                     item = self.create_record(grouped=grouped)
@@ -1019,11 +1015,19 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
                     self.setattr(item, 'id', i)
                     self.setattr(item, 'indent', level_index)
                     self.setattr(item, 'count', level[i])
-                    for agg in aggregates.keys():
+                    for agg, aggv in aggregates.iteritems():
                         # для средних - посчитаем среднее
-                        if aggregates[agg] == 'avg':
-                            self.setattr(item, agg, aggregate_values[
-                                         i][agg] / self.getattr(item, 'count'))
+                        if agg == 'avg':
+                            self.setattr(
+                                item, agg,
+                                aggregate_values[i][agg] / (
+                                    self.getattr(item, 'count'))
+                            )
+                        elif callable(aggv):
+                            self.setattr(
+                                item, agg,
+                                aggv(aggregate_values[i][agg], None)
+                            )
                         else:
                             self.setattr(item, agg, aggregate_values[i][agg])
 
@@ -1075,7 +1079,8 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
                 else:
                     order.append(sort_field)
             sorted_data = self.multikeysort(pre_res, order)
-        # изначальная ошибка была передавать сортировку в dict - надо было сразу в list
+        # изначальная ошибка была передавать сортировку в dict
+        # - надо было сразу в list
         elif isinstance(sorting, list) and sorting:
             order = []
             for sort_field, sort_dir in sorting:
@@ -1093,29 +1098,19 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
             res.append(item)
             index += 1
 
-        # print 'out_data()= total=%s, res_count=%s' % (total_of_level, len(res))
-        # out_cache[cache_key] = (res,total_of_level)
         return res
 
-    def counter(self, grouped, level_index, level_keys, expandedItems):
-        return self.__count_data(grouped, level_index, level_keys, expandedItems)
+    __read_data = reader
 
-    def __count_data(self, grouped, level_index, level_keys, expandedItems):
+    def counter(self, grouped, level_index, level_keys, expandedItems):
         '''
         подсчет количества строк в раскрытом уровне
         '''
-        # построим ключ кэша
-        # cache_key = '%s__%s__%s__%s' % (','.join(grouped), level_index, ','.join(level_keys), add_to_count_key(expandedItems))
-        # if cache_key in count_cache.keys():
-        #    print 'cached count...........'
-        #    return count_cache[cache_key]
-
-        # print 'count_exp_data(): grouped=%s, level_index=%s, level_keys=%s,
-        # expandedItems=%s' % (grouped, level_index, level_keys, expandedItems)
         total_of_level = 0
         if grouped:
             if level_index == 0:
-                # вывести элементы 1-го уровня группировки (не нужно использовать ключи)
+                # вывести элементы 1-го уровня группировки
+                # (не нужно использовать ключи)
                 level = []
                 # переберем элементы и сформируем уровень
                 field = grouped[level_index]
@@ -1127,7 +1122,8 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
             else:
                 # для всех остальных элементов будут использоваться ключи
                 level = []
-                # если берется уровень больший, чем количество группировок, то выдаем просто записи
+                # если берется уровень больший, чем количество группировок,
+                # то выдаем просто записи
                 if level_index >= len(grouped):
                     field = None
                 else:
@@ -1141,7 +1137,8 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
                         # подходит ли запись под группировку
                         if key != key_value:
                             break
-                        # если успешно проверили все поля, то значит это наша запись
+                        # если успешно проверили все поля,
+                        # то значит это наша запись
                         elif lev == level_index - 1:
                             if field:
                                 group_value = self.getattr(rec, field)
@@ -1157,8 +1154,12 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
         exp_count = 0
         for exp in expandedItems:
             if exp['count'] == -1:
-                exp['count'] = self.counter(grouped, level_index + 1, level_keys + [
-                                            exp['id']], exp['expandedItems'])
+                exp['count'] = self.counter(
+                    grouped,
+                    level_index + 1,
+                    level_keys + [exp['id']],
+                    exp['expandedItems']
+                )
             exp_count = exp_count + exp['count']
 
         # count_cache[cache_key] = total_of_level+exp_count
@@ -1166,12 +1167,15 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
         # (total_of_level+exp_count, total_of_level, exp_count)
         return total_of_level + exp_count
 
-    def indexer(self, grouped, level_index, level_keys, expandedItems, aggregates, sorting):
-        return self.__index_data(grouped, level_index, level_keys, expandedItems, aggregates, sorting)
+    __count_data = counter
 
-    def __index_data(self, grouped, level_index, level_keys, expandedItems, aggregates, sorting):
+    def indexer(
+        self, grouped, level_index, level_keys,
+        expandedItems, aggregates, sorting
+    ):
         '''
-        построение индексов элементов в раскрытом уровне, только для группировок и для тех, которые раскрыты
+        построение индексов элементов в раскрытом уровне,
+        только для группировок и для тех, которые раскрыты
         '''
         res = []
         pre_res = []
@@ -1205,26 +1209,11 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
                         level[group_value] = level[group_value] + 1
                         aggr_rec = aggregate_values[group_value]
                     # будем считать агрегаты
-                    for agg in aggregates.keys():
-                        agg_type = aggregates[agg]
-                        agg_value = self.getattr(rec, agg)
-                        if agg_type == 'sum':
-                            aggr_rec[agg] = agg_value + (aggr_rec[
-                                                         agg] if agg in aggr_rec else 0)
-                        elif agg_type == 'count':
-                            aggr_rec[agg] = 1 + (aggr_rec[
-                                                 agg] if agg in aggr_rec else 0)
-                        elif agg_type == 'min':
-                            aggr_rec[agg] = agg_value if aggr_rec.get(
-                                agg, 0) > agg_value else aggr_rec.get(agg, 0)
-                        elif agg_type == 'max':
-                            aggr_rec[agg] = agg_value if aggr_rec.get(
-                                agg, 0) < agg_value else aggr_rec.get(agg, 0)
-                        elif agg_type == 'avg':
-                            aggr_rec[agg] = agg_value + (aggr_rec[
-                                                         agg] if agg in aggr_rec else 0)
+                    self.update_aggregates(aggregates, aggr_rec, rec)
 
-            # придется обработать все записи уровня, т.к. требуется еще отсортировать их и лишь потом ограничить количество
+            # придется обработать все записи уровня,
+            # т.к. требуется еще отсортировать их
+            # и лишь потом ограничить количество
             for i, rec in prepared:
                 item = self.create_record()
                 self.setattr(item, field, i)
@@ -1234,15 +1223,18 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
                 for agg in aggregates.keys():
                     # для средних - посчитаем среднее
                     if aggregates[agg] == 'avg':
-                        self.setattr(item, agg, aggregate_values[
-                                     i][agg] / self.getattr(item, 'count'))
+                        self.setattr(
+                            item, agg, (
+                                aggregate_values[i][agg] /
+                                self.getattr(item, 'count')))
                     else:
                         self.setattr(item, agg, aggregate_values[i][agg])
 
                 # Добавляем в прокси расшифровку для группируемого поля
                 detail_attr = self.detail_attrs_map.get(field)
                 if detail_attr:
-                    self.setattr(item, detail_attr, self.getattr(rec, detail_attr))
+                    self.setattr(
+                        item, detail_attr, self.getattr(rec, detail_attr))
 
                 # проставим значения ключей уровня
                 for lev in range(0, level_index):
@@ -1253,7 +1245,8 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
                     # Добавляем в прокси атрибуты для расшифровки
                     detail_attr = self.detail_attrs_map.get(lev_field)
                     if detail_attr:
-                        self.setattr(item, detail_attr, self.getattr(rec, detail_attr))
+                        self.setattr(
+                            item, detail_attr, self.getattr(rec, detail_attr))
 
                 self.calc(item)
                 pre_res.append(item)
@@ -1268,7 +1261,8 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
                     else:
                         order.append(sort_field)
                 sorted_data = self.multikeysort(pre_res, order)
-            # изначальная ошибка была передавать сортировку в dict - надо было сразу в list
+            # изначальная ошибка была передавать сортировку в dict
+            # - надо было сразу в list
             elif isinstance(sorting, list) and sorting:
                 order = []
                 for sort_field, sort_dir in sorting:
@@ -1284,11 +1278,22 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
                 res.append(self.getattr(item, 'id'))
         return res
 
+    __index_data = indexer
+
     def multikeysort(self, items, columns):
-        comparers = [ ((col[1:].strip(), -1) if col.startswith('-') else (col.strip(), 1)) for col in columns]
+        comparers = [
+            (
+                (col[1:].strip(), -1)
+                if col.startswith('-') else
+                (col.strip(), 1)
+            ) for col in columns
+        ]
         def comparer(left, right):
             for attr, mult in comparers:
-                result = cmp(self.getattr(left, attr), self.getattr(right, attr))
+                result = cmp(
+                    self.getattr(left, attr),
+                    self.getattr(right, attr)
+                )
                 if result:
                     return mult * result
             else:
